@@ -1,4 +1,4 @@
-/* © 2006-2007 Anselm R. Garbe <garbeam at gmail dot com>
+/* © 2006-2008 Anselm R Garbe <garbeam at gmail dot com>
  * See LICENSE file for license details. */
 #define _XOPEN_SOURCE 500
 #if HAVE_SHADOW_H
@@ -16,6 +16,12 @@
 #include <X11/keysym.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/extensions/dpms.h>
+
+#if HAVE_BSD_AUTH
+#include <login_cap.h>
+#include <bsd_auth.h>
+#endif
 
 void
 eprint(const char *errstr, ...) {
@@ -27,6 +33,7 @@ eprint(const char *errstr, ...) {
 	exit(EXIT_FAILURE);
 }
 
+#ifndef HAVE_BSD_AUTH
 const char *
 get_password() { /* only run as root */
 	const char *rval;
@@ -46,18 +53,23 @@ get_password() { /* only run as root */
 		rval = sp->sp_pwdp;
 	}
 #endif
+
 	/* drop privileges */
 	if(setgid(pw->pw_gid) < 0 || setuid(pw->pw_uid) < 0)
 		eprint("slock: cannot drop privileges\n");
 	return rval;
 }
+#endif
 
 int
 main(int argc, char **argv) {
 	char curs[] = {0, 0, 0, 0, 0, 0, 0, 0};
 	char buf[32], passwd[256];
 	int num, screen;
+
+#ifndef HAVE_BSD_AUTH
 	const char *pws;
+#endif
 	unsigned int len;
 	Bool running = True;
 	Cursor invisible;
@@ -69,9 +81,15 @@ main(int argc, char **argv) {
 	XEvent ev;
 	XSetWindowAttributes wa;
 
-	if((argc > 1) && !strncmp(argv[1], "-v", 3))
-		eprint("slock-"VERSION", © 2006-2007 Anselm R. Garbe, Sander van Dijk\n");
+	if((argc == 2) && !strcmp("-v", argv[1]))
+		eprint("slock-"VERSION", © 2006-2008 Anselm R Garbe\n");
+	else if(argc != 1)
+		eprint("usage: slock [-v]\n");
+
+#ifndef HAVE_BSD_AUTH
 	pws = get_password();
+#endif
+
 	if(!(dpy = XOpenDisplay(0)))
 		eprint("slock: cannot open display\n");
 	screen = DefaultScreen(dpy);
@@ -107,7 +125,9 @@ main(int argc, char **argv) {
 	XSync(dpy, False);
 
 	/* main event loop */
-	while(running && !XNextEvent(dpy, &ev))
+	while(running && !XNextEvent(dpy, &ev)) {
+		if(len == 0) 
+			DPMSForceLevel(dpy, DPMSModeOff);
 		if(ev.type == KeyPress) {
 			buf[0] = 0;
 			num = XLookupString(&ev.xkey, buf, sizeof buf, &ksym, 0);
@@ -118,7 +138,14 @@ main(int argc, char **argv) {
 			switch(ksym) {
 			case XK_Return:
 				passwd[len] = 0;
-				if((running = strcmp(crypt(passwd, pws), pws)) != 0)
+
+#ifdef HAVE_BSD_AUTH
+				running = !auth_userokay(getlogin(), NULL, "auth-xlock", passwd);
+#else
+				running = strcmp(crypt(passwd, pws), pws);
+#endif
+
+				if (running != 0)
 					XBell(dpy, 100);
 				len = 0;
 				break;
@@ -137,6 +164,7 @@ main(int argc, char **argv) {
 				break;
 			}
 		}
+	}
 	XUngrabPointer(dpy, CurrentTime);
 	XFreePixmap(dpy, pmap);
 	XDestroyWindow(dpy, w);
